﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.MyMath;
using TestView.Condicionador.ItemCalibracao;
using System.Runtime.Serialization;
using System.IO;
using TestView.Serializacao;

namespace TestView.Condicionador
{
    [DataContract]
    [KnownType(typeof(PontoDeCalibracao))]
    public class Calibracao
    {
        #region Constantes

        public const int ResolucaoPadrao = 1;
        public const int TransdutorPadrao = 1;
        public const int PontoDecimalPadrao = 2;
        public const double CapacidadePadrao = 1000.0;
        public const double TaraPadrao = 0.0;

        #endregion

        #region Construtor e membros

        public Calibracao(Transdutor tipoTransdutor, int resolucao, int pontoDecimal, Unidade unidade, double capacidade)
        {
            this.PontoDecimal = pontoDecimal;
            this.Resolucao = resolucao;
            this.Unidade = unidade;
            this.Tara = TaraPadrao;
            this.TipoTransdutor = tipoTransdutor;
            this.Capacidade = capacidade;
            this._pontosDeCalibracao = new List<PontoDeCalibracao>();
            this.InverterSinal = false;
            this.Interpolacao = new InterpolacaoLinear(this._pontosDeCalibracao);
        }

        public Calibracao()
        {
            this.reiniciaCalibracao();
        }

        private List<PontoDeCalibracao> _pontosDeCalibracao;
        private double _tara;
        private double _capacidade;
        private Transdutor _tipoTransdutor;
        private int _resolucao;
        private int _pontoDecimal;
        private Unidade _unidade;
        private Interpolacao _interpolacao;
        private bool _inverterSinal;

        [DataMember]
        public List<PontoDeCalibracao> PontosDeCalibracao
        {
            get
            {
                return new List<PontoDeCalibracao>(this._pontosDeCalibracao);
            }
            set
            {
                this._pontosDeCalibracao = value;
            }
        }

        [DataMember]
        public double Tara
        {
            get
            {
                return this._tara;
            }
            set
            {
                this._tara = value;
                this.informParametrosDeCalibracaoListeners();
            }
        }

        [DataMember]
        public double Capacidade
        {
            get
            {
                return this._capacidade;
            }
            set
            {
                this._capacidade = value;
                this.informParametrosDeCalibracaoListeners();
            }
        }

        [DataMember]
        public Transdutor TipoTransdutor
        {
            get
            {
                return this._tipoTransdutor;
            }
            set
            {
                this._tipoTransdutor = Transdutor.MapaTransdutores[value.ToString()];
                this.informParametrosDeCalibracaoListeners();
            }
        }

        [DataMember]
        public int Resolucao
        {
            get
            {
                return this._resolucao;
            }
            set
            {
                this._resolucao = value;
                this.informParametrosDeCalibracaoListeners();
            }
        }

        [DataMember]
        public int PontoDecimal
        {
            get
            {
                return this._pontoDecimal;
            }
            set
            {
                this._pontoDecimal = value;
                this.informParametrosDeCalibracaoListeners();
            }
        }

        [DataMember]
        public Unidade Unidade
        {
            get
            {
                return this._unidade;
            }
            set
            {
                this._unidade = Unidade.MapaUnidades[value.Id];
                this.informParametrosDeCalibracaoListeners();
            }
        }

        [DataMember]
        public bool InverterSinal
        {
            get
            {
                return this._inverterSinal;
            }
            set
            {
                this._inverterSinal = value;
            }
        }

        public Interpolacao Interpolacao
        {
            get
            {
                return this._interpolacao;
            }
            set
            {
                this._interpolacao = value;
            }
        }

        #endregion

        #region Calibracao

        public void reiniciaCalibracao()
        {
            this.PontoDecimal = PontoDecimalPadrao;
            this.Resolucao = ResolucaoPadrao;
            this.Unidade = Unidade.KGF;
            this.Tara = TaraPadrao;
            this.TipoTransdutor = Transdutor.FORCA;
            this.Capacidade = CapacidadePadrao;
            this._pontosDeCalibracao = new List<PontoDeCalibracao>();
            this.Interpolacao = new InterpolacaoLinear(this._pontosDeCalibracao);
        }

        public void adicionaPontoCalibracao(long ponto, double valor)
        {
            this._pontosDeCalibracao.Add(new PontoDeCalibracao(ponto, valor));
            this._pontosDeCalibracao.Sort();
            this.informPontosDeCalibracaoListeners();
        }

        public void removePontoCalibracao(int indice)
        {
            PontoDeCalibracao ponto = this._pontosDeCalibracao[indice];
            this.removePontoCalibracao(ponto);
        }

        public void removePontoCalibracao(PontoDeCalibracao pontoCalibracao)
        {
            this._pontosDeCalibracao.Remove(pontoCalibracao);
            this._pontosDeCalibracao.Sort();
            this.informPontosDeCalibracaoListeners();
        }

        public void atualizaPontos(List<PontoDeCalibracao> pontos)
        {
            this._pontosDeCalibracao.Clear();
            this._pontosDeCalibracao.AddRange(pontos);
            this._pontosDeCalibracao.Sort();
            this.informPontosDeCalibracaoListeners();
        }

        public void tara(long indicacao)
        {
            double indicacaoInterpolada = this.Interpolacao.obtemValorOrdenada(indicacao);
            this.Tara = indicacaoInterpolada;
        }

        public void inverterSinal()
        {
            this.InverterSinal = !this.InverterSinal;
        }

        #endregion

        #region Indicacao

        public double obtemIndicacaoAjustada(long indicacao)
        {
            double valorInterpolado = this.Interpolacao.obtemValorOrdenada(indicacao);
            double indicacaoAjustada = valorInterpolado - this.Tara;
            if (this.InverterSinal)
            {
                indicacaoAjustada *= -1.0;
            }
            indicacaoAjustada = Convert.ToInt32(indicacaoAjustada / (this.Resolucao * Math.Pow(10.0, -this.PontoDecimal))) * (this.Resolucao * Math.Pow(10.0, -this.PontoDecimal));
            return indicacaoAjustada;
        }

        #endregion

        #region Eventos

        #region PontosDeCalibracaoChanged

        public delegate void PontosDeCalibracaoChangedEvent(object sender, PontosDeCalibracaoEventArgs e);

        public event PontosDeCalibracaoChangedEvent PontosDeCalibracaoChanged;

        public class PontosDeCalibracaoEventArgs : EventArgs
        {
            public PontosDeCalibracaoEventArgs(List<PontoDeCalibracao> pontos)
            {
                this.PontosDeCalibracao = pontos;
            }

            public List<PontoDeCalibracao> PontosDeCalibracao
            {
                get;
                set;
            }
        }

        private void informPontosDeCalibracaoListeners()
        {
            if (PontosDeCalibracaoChanged != null)
            {
                PontosDeCalibracaoChanged(this, new PontosDeCalibracaoEventArgs(this.PontosDeCalibracao));
            }
        }

        #endregion

        #region ParametrosDeCalibracaoChanged

        public delegate void ParametrosDeCalibracaoChangedEvent(object sender, ParametrosDeCalibracaoEventArgs e);

        public event ParametrosDeCalibracaoChangedEvent ParametrosDeCalibracaoChanged;

        public class ParametrosDeCalibracaoEventArgs : EventArgs
        {
            public ParametrosDeCalibracaoEventArgs(double tara, int resolucao, int pontoDecimal, Unidade unidade, Transdutor tipoTransdutor, double capacidade)
            {
                this.Unidade = unidade;
                this.Resolucao = resolucao;
                this.Tara = tara;
                this.TipoTransdutor = tipoTransdutor;
                this.Capacidade = capacidade;
                this.PontoDecimal = pontoDecimal;
            }

            public double Tara { get; set; }
            public int Resolucao { get; set; }
            public int PontoDecimal { get; set; }
            public Unidade Unidade { get; set; }
            public Transdutor TipoTransdutor { get; set; }
            public double Capacidade { get; set; }
        }

        private void informParametrosDeCalibracaoListeners()
        {
            if (ParametrosDeCalibracaoChanged != null)
            {
                ParametrosDeCalibracaoChanged(this, new ParametrosDeCalibracaoEventArgs(this.Tara, this.Resolucao, this.PontoDecimal, this.Unidade, this.TipoTransdutor, this.Capacidade));
            }
        }

        #endregion

        #endregion

        #region Serializacao

        [OnDeserialized]
        private void posDeserializacao(StreamingContext context)
        {
            this.Interpolacao = new InterpolacaoLinear(this._pontosDeCalibracao);
        }

        #endregion

        #region Misc

        public Calibracao Clone()
        {
            Calibracao clone = new Calibracao(this.TipoTransdutor, this.Resolucao, this.PontoDecimal, this.Unidade, this.Capacidade);
            return clone;
        }

        public static int[] listaResolucao()
        {
            return new int[] { 1, 2, 5, 10, 20, 50, 100 };
        }

        public static Dictionary<Transdutor, string> dicionarioTipoTransdutor()
        {
            Dictionary<Transdutor, string> dicionario = new Dictionary<Transdutor, string>();
            dicionario.Add(Transdutor.FORCA, "Transdutor de força");
            dicionario.Add(Transdutor.DEFORMACAO, "Transdutor de deformação");
            dicionario.Add(Transdutor.DESLOCAMENTO, "Transdutor de deslocamento");
            return dicionario;
        }

        #endregion
    }
}
