﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Condicionador;
using System.Runtime.Serialization;
using TestView.Indicacao;
using TestView.Seguranca;
using TestView.Ensaios.Utilidades.Instrumentacao;

namespace TestView.Conexoes
{
    [DataContract]
    [KnownType(typeof(Extensometro))]
    [KnownType(typeof(EncoderEletroMecanico))]
    [KnownType(typeof(InstrumentoNaoEspecificado))]
    public class Conexao
    {
        #region Contrutor e membros

        private PortaSerial _portaSerial;
        private Indicador _indicador;
        private CondicionadorSinal _condicionadorDeSinal;
        private Instrumento _instrumento;

        [DataMember]
        public PortaSerial PortaSerial 
        { 
            get
            {
                return this._portaSerial;
            }
            set
            {
                this._portaSerial = value;
            }
        }
        [DataMember]
        public CondicionadorSinal CondicionadorSinal 
        {
            get
            {
                return this._condicionadorDeSinal;
            }
            set
            {
                this._condicionadorDeSinal = value;
            }
        }
        [DataMember]
        public Instrumento Instrumento
        {
            get
            {
                return this._instrumento;
            }
            set
            {
                this._instrumento = value;
                this.informInstrumentoListeners();
            }
        }

        public Indicador Indicador
        {
            get 
            {
                return this._indicador;
            }
        }

        public bool Conectado
        {
            get
            {
                return this.PortaSerial.estaConectado();
            }
        }

        public Conexao(String nomePorta, int modeloCondicionadorSinal, int baudRate)
        {
            this.PortaSerial = new PortaSerial(nomePorta, baudRate);
            this.CondicionadorSinal = new CondicionadorSinal(modeloCondicionadorSinal);
            this.Instrumento = new InstrumentoNaoEspecificado();
            this._indicador = new Indicador(this);
            this.Indicador.IndicacaoChanged += new Indicacao.Indicador.IndicacaoChangedEvent(indicacaoChanged);
        }

        # endregion

        #region Conexão

        public void conectar()
        {
            try
            {
                this.PortaSerial.conectar();
                this.carregaConfiguracao();
                this.informConexaoListeners();
            }
            catch (TimeoutException ex)
            {
                this.desconectar();
                String mensagem = String.Format(Properties.Resources.timeoutConexaoString, this.PortaSerial.NomePorta);
                Logging.getInstance().error(mensagem, ex, true);
            }
        }

        public void desconectar()
        {
            lock (this)
            {
                this.PortaSerial.desconectar();
                this.informConexaoListeners();
            }
        }

        #endregion

        #region Eventos

        #region IndicadorChanged

        void indicacaoChanged(object sender, Indicador.IndicacaoEventArgs e)
        {
            this.informIndicadorListeners(e.Indicacao, e.IndicacaoAjustada, e.IndicacaoAjustadaStr, e.IndicacaoRelativa);
        }

        public delegate void IndicacaoChangedEvent(object sender, TestView.Indicacao.Indicador.IndicacaoEventArgs e);

        public event IndicacaoChangedEvent IndicacaoChanged;

        private void informIndicadorListeners(long indicacao, double indicacaoAjustada, string indicacaoAjustadaStr, double indicacaoRelativa)
        {
            if (IndicacaoChanged != null)
            {
                IndicacaoChanged(this, new TestView.Indicacao.Indicador.IndicacaoEventArgs(indicacao, indicacaoAjustada, indicacaoAjustadaStr, indicacaoRelativa));
            }
        }

        #endregion

        #region ConexaoChanged

        public delegate void ConexaoChangedEvent(object sender, ConexaoEventArgs e);

        public event ConexaoChangedEvent ConexaoChanged;

        public class ConexaoEventArgs : EventArgs
        {
            public ConexaoEventArgs(bool conectado)
            {
                this.EstaConectado = conectado;
            }

            public bool EstaConectado
            {
                get;
                set;
            }
        }

        private void informConexaoListeners()
        {
            if (ConexaoChanged != null)
            {
                ConexaoChanged(this, new ConexaoEventArgs(this.Conectado));
            }
        }

        #endregion

        #region InstrumentoChanged

        public delegate void InstrumentoChangedEvent(object sender, InstrumentoChangedEventArgs e);

        public event InstrumentoChangedEvent InstrumentoChanged;

        public class InstrumentoChangedEventArgs : EventArgs
        {
            public InstrumentoChangedEventArgs(Instrumento instrumento)
            {
                this.Instrumento = instrumento;
            }

            public Instrumento Instrumento
            {
                get;
                set;
            }
        }

        private void informInstrumentoListeners()
        {
            if (InstrumentoChanged != null)
            {
                InstrumentoChanged(this, new InstrumentoChangedEventArgs(this.Instrumento));
            }
        }

        #endregion

        #endregion

        #region Comandos

        public String setFiltroDigital(int indice)
        {
            lock (this)
            {
                String resposta = null;
                this.PortaSerial.envia(Comando.setAsf(indice));
                resposta = this.PortaSerial.recebe(3);
                return resposta;
            }
        }

        public int getFiltroDigital()
        {

            try
            {
                lock (this)
                {
                    String resposta = null;
                    this.PortaSerial.envia(Comando.getAsf());
                    resposta = this.PortaSerial.recebe(4);
                    return Int32.Parse(resposta);
                }
            }
            catch (TimeoutException ex)
            {
                this.desconectar();
                throw ex;
            }
        }

        public long getIndicacao()
        {
            try
            {
                lock (this)
                {
                    String resposta = null;
                    int numeroIndicacoes = 1;
                    this.PortaSerial.envia(Comando.getMsv(numeroIndicacoes));
                    resposta = this.PortaSerial.recebe(numeroIndicacoes * 10);
                    return long.Parse(resposta);
                }
            }
            catch (TimeoutException ex)
            {
                this.desconectar();
                throw ex;
            }
            catch (FormatException ex)
            {
                throw ex;
            }
        }

        public void tara()
        {
            if (this.Conectado && this.Indicador.Inicializado)
            {
                long ultimaIndicacao = this.Indicador.Indicacao;
                this.CondicionadorSinal.Calibracao.tara(ultimaIndicacao);
            }
        }

        public void inverterSinal()
        {
            this.CondicionadorSinal.Calibracao.inverterSinal();
        }

        #endregion

        #region Dados

        private void carregaConfiguracao()
        {
            this.CondicionadorSinal.Configuracao.FiltroDigital = this.getFiltroDigital();
        }

        #endregion

        #region Misc

        public override string ToString()
        {
            String nomeConexao = "";
            nomeConexao += this.PortaSerial.NomePorta;
            nomeConexao += " - ";
            nomeConexao += this.CondicionadorSinal.Modelo;
            return nomeConexao;
        }

        #endregion

        #region Serializacao

        [OnDeserialized]
        private void inicializaIndicador(StreamingContext context)
        {
            this._indicador = new Indicador(this);
            this.Indicador.IndicacaoChanged += new Indicacao.Indicador.IndicacaoChangedEvent(indicacaoChanged);
        }

        #endregion
    }
}
