﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Conexoes;
using System.Runtime.Serialization;
using System.IO;
using TestView.Serializacao;
using TestView.Indicacao;
using TestView.Seguranca;
using TestView.Ensaios;
using TestView.Ensaios.Modulos;
using TestView.Ensaios.Utilidades.Instrumentacao;
using System.Reflection;
using TestView.Dados;

namespace TestView
{
    [DataContract]
    public class GerenciadorEnsaios
    {
        #region Construtor e membros

        private const string ARQUIVO_CONFIGURACAO_GERENCIADOR = "./opcoes_gerais.xml";

        private List<Conexao> _conexoes;
        public List<IModuloDeEnsaio> _ensaios;
        private static GerenciadorEnsaios _instance;

        public String Versao
        {
            get
            {
                return Assembly.GetExecutingAssembly().GetName().Version.ToString();
            }
        }
        [DataMember]
        public List<Conexao> Conexoes
        {
            get
            {
                return this._conexoes;
            }
            set
            {
                this._conexoes = value;
            }
        }
        public List<IModuloDeEnsaio> Ensaios
        {
            get
            {
                return this._ensaios;
            }
        }
        public static GerenciadorEnsaios getInstance()
        {
            if (_instance == null)
            {
                _instance = new GerenciadorEnsaios();
            }
            return _instance;
        }

        private GerenciadorEnsaios()
        {
            this._conexoes = new List<Conexao>();
            this._ensaios = new List<IModuloDeEnsaio>();
            this.carregaArquivoConfiguracao();
            foreach (Conexao conexao in this._conexoes)
            {
                conexao.CondicionadorSinal.carregarArquivoDeCalibracao();
            }
            this.carregaModulos();
        }

        #endregion

        #region Encerramento

        public void encerrar()
        {
            foreach (Conexao conexao in this.listaConexoes())
            {
                this.desconectar(conexao);
            }
            GerenciadorEnsaios.getInstance().salvaArquivoConfiguracao();
        }

        #endregion

        #region Conexoes

        public void removeTodasConexoes()
        {
            this._conexoes.Clear();
            this.informListaConexoesListeners();
        }

        public void removeConexao(Conexao conexao)
        {
            this._conexoes.Remove(conexao);
            this.informListaConexoesListeners();
        }

        public Conexao adicionaNovaConexao(String nomePorta, int condicionadorSinal)
        {
            return adicionaNovaConexao(nomePorta, condicionadorSinal, PortaSerial.BaudRatePadrao);
        }

        public Conexao adicionaNovaConexao(String nomePorta, int condicionadorSinal, int baudrate)
        {
            Conexao novaConexao = new Conexao(nomePorta, condicionadorSinal, baudrate);
            this._conexoes.Add(novaConexao);
            this.informListaConexoesListeners();
            return novaConexao;
        }

        public void alternarEstadoConexao(Conexao conexao)
        {
            if (conexao.Conectado)
            {
                this.desconectar(conexao);
            }
            else
            {
                this.conectar(conexao);
            }
        }

        public void conectar(Conexao conexao)
        {
            Logging.getInstance().debug(String.Format("Conectar {0}", conexao), false);
            conexao.conectar();
            conexao.CondicionadorSinal.Calibracao.reiniciaCalibracao();
            conexao.CondicionadorSinal.carregarArquivoDeCalibracao();
            conexao.Indicador.iniciaIndicador();
        }

        public void zerar(Conexao conexao)
        {
            conexao.tara();
        }

        public void inverterSinal(Conexao conexao)
        {
            conexao.inverterSinal();
        }

        public void desconectar(Conexao conexao)
        {
            Logging.getInstance().debug(String.Format("Desconectar {0}", conexao), false);
            conexao.Indicador.paraIndicador();
            conexao.desconectar();
        }

        public List<Conexao> listaConexoes()
        {
            return this._conexoes;
        }

        #endregion

        #region Arquivo de configuração

        public void salvaArquivoConfiguracao(string nomeDoArquivo)
        {
            String caminhoCompleto = Path.GetFullPath(nomeDoArquivo);
            Stream stream = new FileStream(caminhoCompleto, FileMode.Create);
            Serializador.serializaObjeto(stream, this, typeof(GerenciadorEnsaios));
            stream.Close();
        }

        public void salvaArquivoConfiguracao()
        {
            this.salvaArquivoConfiguracao(ARQUIVO_CONFIGURACAO_GERENCIADOR);
        }

        public void carregaArquivoConfiguracao(string nomeDoArquivo)
        {
            if (File.Exists(nomeDoArquivo))
            {
                String caminhoCompleto = Path.GetFullPath(nomeDoArquivo);
                Stream stream = new FileStream(caminhoCompleto, FileMode.Open);
                GerenciadorEnsaios gerenciador = (GerenciadorEnsaios)Serializador.deserializaObjeto(stream, typeof(GerenciadorEnsaios));
                this.Conexoes = gerenciador.Conexoes;
                stream.Close();
            }
        }

        public void carregaArquivoConfiguracao()
        {
            this.carregaArquivoConfiguracao(ARQUIVO_CONFIGURACAO_GERENCIADOR);
        }

        #endregion

        #region Ensaios

        public void novoEnsaio(BibliotecaModulo modulo)
        {
            IModuloDeEnsaio moduloEnsaio = modulo.obtemNovaInstanciaModulo();
            this._ensaios.Add(moduloEnsaio);
            Logging.getInstance().info(String.Format("Adicionado novo ensaio: {0}", modulo), false);
            this.informEnsaioAdicionadoListeners(moduloEnsaio);
        }

        public void abrirArquivoDeEnsaio(String nomeArquivo)
        {
            Stream stream = new FileStream(nomeArquivo, FileMode.Open);
            Ensaio ensaio = null;
            IModuloDeEnsaio modulo = null;
            foreach (BibliotecaModulo biblioteca in CarregadorDeModulos.getInstance().BibliotecasCarregadas)
            {
                try
                {
                    ensaio = biblioteca.obtemNovaInstanciaEnsaio().deserializaEnsaio(stream);
                    modulo = biblioteca.obtemNovaInstanciaModulo();
                }
                catch (Exception ex)
                {
                    Logging.getInstance().error(ex.Message, ex, false);
                }
            }
            stream.Close();
            if (ensaio == null || modulo == null)
            {
                Logging.getInstance().error(Properties.Resources.impossivelAbrirArquivoEnsaioString, true);
            }
            else
            {
                this.abrirEnsaio(ensaio, modulo);
            }
        }

        public void abrirEnsaio(Ensaio ensaio, IModuloDeEnsaio modulo)
        {
            ensaio.EstadoEnsaio = EstadoEnsaio.FECHADO;
            modulo.RotinaDeEnsaio.abrirEnsaio(ensaio);
            this._ensaios.Add(modulo);
            this.informEnsaioAdicionadoListeners(modulo);
        }

        public void salvarArquivoEnsaio(String nomeArquivo, IModuloDeEnsaio moduloEnsaio)
        {
            Ensaio ensaio = moduloEnsaio.RotinaDeEnsaio.Ensaio;
            if (ensaio.EstadoEnsaio == EstadoEnsaio.INICIADO || ensaio.EstadoEnsaio == EstadoEnsaio.NAO_INICIADO)
            {
                Logging.getInstance().warn(Properties.Resources.impossivelSalvarEnsaioNaoTerminadoString, true);
            }
            else
            {
                Stream stream = new FileStream(nomeArquivo, FileMode.Create);
                ensaio.serializaEnsaio(stream);
                stream.Close();
            }
        }

        public void removeEnsaio(IModuloDeEnsaio moduloEnsaio)
        {
            moduloEnsaio.RotinaDeEnsaio.terminaEnsaioEspecifico();
            this._ensaios.Remove(moduloEnsaio);
            this.informEnsaioRemovidoListeners(moduloEnsaio);
        }

        #endregion

        #region Modulos

        private void carregaModulos()
        {
            CarregadorDeModulos.getInstance().carregaModulos();
        }

        #endregion

        #region Eventos

        #region ListaConexoesChanged

        public delegate void ListaConexoesChangedEvent(object sender, ListaConexoesEventArgs e);

        public event ListaConexoesChangedEvent ListaConexoesChanged;

        public class ListaConexoesEventArgs : EventArgs
        {
            public ListaConexoesEventArgs(List<Conexao> conexoes)
            {
                this.ListaConexoes = conexoes;
            }

            public List<Conexao> ListaConexoes
            {
                get;
                set;
            }
        }

        private void informListaConexoesListeners()
        {
            if (ListaConexoesChanged != null)
            {
                ListaConexoesChanged(this, new ListaConexoesEventArgs(this._conexoes));
            }
        }

        #endregion

        #region EnsaioAdicionado

        public delegate void EnsaioAdicionadoEvent(object sender, EnsaioAdicionadoEventArgs e);

        public event EnsaioAdicionadoEvent ModuloEnsaioAdicionado;

        public class EnsaioAdicionadoEventArgs : EventArgs
        {
            public EnsaioAdicionadoEventArgs(IModuloDeEnsaio ensaio)
            {
                this.ModuloEnsaio = ensaio;
            }

            public IModuloDeEnsaio ModuloEnsaio
            {
                get;
                set;
            }
        }

        private void informEnsaioAdicionadoListeners(IModuloDeEnsaio ensaio)
        {
            if (ModuloEnsaioAdicionado != null)
            {
                ModuloEnsaioAdicionado(this, new EnsaioAdicionadoEventArgs(ensaio));
            }
        }

        #endregion

        #region EnsaioRemovido

        public delegate void EnsaioRemovidoEvent(object sender, EnsaioRemovidoEventArgs e);

        public event EnsaioRemovidoEvent ModuloEnsaioRemovido;

        public class EnsaioRemovidoEventArgs : EventArgs
        {
            public EnsaioRemovidoEventArgs(IModuloDeEnsaio ensaio)
            {
                this.ModuloEnsaio = ensaio;
            }

            public IModuloDeEnsaio ModuloEnsaio
            {
                get;
                set;
            }
        }

        private void informEnsaioRemovidoListeners(IModuloDeEnsaio ensaio)
        {
            if (ModuloEnsaioRemovido != null)
            {
                ModuloEnsaioRemovido(this, new EnsaioRemovidoEventArgs(ensaio));
            }
        }

        #endregion

        #endregion
    }
}
